@@ -245,7 +245,7 @@ class MainWindow(UberSquareWindow):
 		if (dialog.result() != QDialog.Accepted):
 			return None
 
-		venueName = dialog.text()
+		venueName = dialog.text().encode('utf-8')
 		categoryId = dialog.category()
 		ll = LocationProvider().get_ll()
 
